from abc import abstractmethod
from logging import Logger
from typing import Protocol, runtime_checkable

from ..base.plugins_meta.base_plugins import BaseEventPlugin


@runtime_checkable
class Event(Protocol):
    def asDict(self):
        ...


@runtime_checkable
class RawEvent(Protocol):
    def asHandlerEventDict(self):
        ...


class BaseEventSendingPlugin(BaseEventPlugin):
    """Base class for event sending."""

    # event name for triggering callback
    eventName = "sending_event"

    @abstractmethod
    async def sendEvents(
        self,
        events: list[Event | RawEvent],
        handlerId: str,
        accountId: str,
        requestId: str,
        createEventTime: str,
        endEventTime: str,
        logger: Logger,
    ) -> None:
        """
        Callback that is triggered on every success request to handlers.

        Args:
            events: event list
            handlerId: handler id
            accountId: account id
            requestId: request id
            createEventTime: event creation time
            endEventTime: event end time
            logger: logger

        """

    async def handleEvent(
        self,
        events: list[Event | RawEvent],
        handlerId: str,
        accountId: str,
        requestId: str,
        createEventTime: str,
        endEventTime: str,
        logger: Logger,
    ):
        """
        Handle events.
        Args:
            events: event list
            handlerId: handler id
            accountId: account id
            requestId: request id
            createEventTime: event creation time
            endEventTime: event end time
            logger: logger

        """
        await self.sendEvents(events, handlerId, accountId, requestId, createEventTime, endEventTime, logger)
